@@ -157,16 +157,17 @@ def AddBaseArgs(parser):
   flags.AddStorageProvisionedThroughput(parser)
   flags.AddInstanceType(parser)
   flags.AddNodeCount(parser)
-  flags.AddActiveDirectoryMode(parser, hidden=True)
-  flags.AddActiveDirectorySecretManagerKey(parser, hidden=True)
-  flags.AddActiveDirectoryOrganizationalUnit(parser, hidden=True)
-  flags.AddActiveDirectoryDNSServers(parser, hidden=True)
-  flags.ClearActiveDirectoryDNSServers(parser, hidden=True)
+  flags.AddActiveDirectoryMode(parser)
+  flags.AddActiveDirectorySecretManagerKey(parser)
+  flags.AddActiveDirectoryOrganizationalUnit(parser)
+  flags.AddActiveDirectoryDNSServers(parser)
+  flags.ClearActiveDirectoryDNSServers(parser)
   flags.AddForceSqlNetworkArchitecture(parser)
   flags.AddFinalBackup(parser)
   flags.AddFinalbackupRetentionDays(parser)
   flags.AddEnableConnectionPooling(parser)
   flags.AddConnectionPoolFlags(parser)
+  flags.AddReadPoolAutoScaleConfig(parser)
 
   # When adding a new field for instance creation, determine if it should also
   # be included in the restore to new instance command. This command uses backup
@@ -184,11 +185,15 @@ def AddBetaArgs(parser):
   flags.AddReplicationLagMaxSecondsForRecreate(parser)
   flags.AddEnableDbAlignedAtomicWrites(parser)
   flags.AddEnableAcceleratedReplicaMode(parser)
+  flags.AddDataApiAccess(parser)
+  flags.AddServerCertificateRotationMode(parser)
+  flags.AddPerformanceCaptureConfig(parser, hidden=True)
+  flags.AddUncMappings(parser)
 
 
-def AddAlphaArgs(parser):
+def AddAlphaArgs(unused_parser):
   """Declare alpha flags for this command parser."""
-  flags.AddReadPoolAutoScaleConfig(parser, hidden=True)
+  pass
 
 
 def RunBaseCreateCommand(args, release_track):
@@ -381,6 +386,15 @@ def RunBaseCreateCommand(args, release_track):
         ' for more details.'
     )
 
+  if (
+      args.IsKnownAndSpecified('performance_capture_config')
+      and args.IsSpecified('database_version')
+      and not args.database_version.startswith('MYSQL')
+  ):
+    raise sql_exceptions.ArgumentError(
+        '`--performance-capture-config` is only supported for MySQL.'
+    )
+
   if args.IsSpecified('edition') and args.edition == 'enterprise-plus':
     # This logic is awkward because we only want to change behavior for "new"
     # database types to avoid breaking changes.
